#include "menudrop.h"
#include "menuseparator.h"

MenuDrop::MenuDrop() :
    _is_locked(false),
    _button_height(27),
    _button_index(0)
{
    setVisibility(false);
    _content_rect.setFillColor(sf::Color(200, 200, 200));
}

void MenuDrop::input(const sf::Event& event)
{
    if (!_is_visible)
        return;

    Widget::input(event);

    switch (event.type)
    {
    default:
        break;

    case sf::Event::MouseButtonReleased:
    case sf::Event::MouseMoved:
        if (!isUnderMouse(event.mouseMove.x, event.mouseMove.y))
        {
            if (!isLocked() && !hasActiveSubmenus())
                setVisibility(false);
        }
        break;
    }
}

bool MenuDrop::hasActiveSubmenus() const
{
    return std::any_of(_submenus.begin(), _submenus.end(),
                       [](const auto& submenu) -> bool
                       {
                           return submenu->_is_visible;
                       });
}

void MenuDrop::update()
{
    Widget::update();
}

void MenuDrop::draw(sf::RenderTarget& target, sf::RenderStates states) const
{
    if (_is_visible)
        target.draw(_content_rect);

    Widget::draw(target, states);
}

void MenuDrop::setRect(const sf::FloatRect& rect)
{
    _content_rect.setPosition(rect.left, rect.top);
    _content_rect.setSize({rect.width, rect.height});
}

void MenuDrop::setPosition(const sf::Vector2f& position)
{
    _content_rect.setPosition(position);
}

bool MenuDrop::isUnderMouse(int mouse_x, int mouse_y) const
{
    return _is_visible && _content_rect.getGlobalBounds().contains(mouse_x, mouse_y);
}

void MenuDrop::setVisibility(bool is_visible)
{
    Widget::setVisibility(is_visible);

    if (!is_visible)
        _is_locked = false;
}

void MenuDrop::addPushButton(const std::shared_ptr<PushButton>& button)
{
    add(button);
}

void MenuDrop::addCascadeButton(const std::shared_ptr<CascadeMenuButton>& button)
{
    auto& submenu = button->submenu();
    submenu->setParent(_parent);
    _submenus.emplace_back(submenu);
    add(button);
}

void MenuDrop::addSeparator()
{
    add(std::make_shared<MenuSeparator>());
}

void MenuDrop::add(const std::shared_ptr<Widget> &widget)
{
    const auto& parent_size = _content_rect.getSize();
    const auto& parent_position = _content_rect.getPosition();

    widget->setRect(sf::FloatRect(parent_position.x,
                                  parent_position.y + (_button_height * _button_index),
                                  parent_size.x,
                                 _button_height));

    addChild(widget);
    ++_button_index;
}

void MenuDrop::lock()
{
    _is_locked = true;
}

void MenuDrop::unlock()
{
    _is_locked = false;
}

bool MenuDrop::isLocked() const
{
    return _is_locked;
}

